/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2006
 *
 *    File name   : STR91x_Vic.c
 *    Description : Define API for VIC
 *
 *    History :
 *    1. Data        : February 10, 2006
 *       Author      : Stanimir Bonev
 *       Description : Create
 *
 *    $Revision: 1.0 $
 **************************************************************************/
#include "STR91x_Vic.h"

/*************************************************************************
 * Function Name: VIC_SetProtectionMode
 * Parameters: VicController_t VicNumb,
 *             Vic_ProtectionMode_t ProtectionType
 * Return: None
 *
 * Description: According to the parameter - ProtectionType to decide the
 *              VIC access mode (previledged mode | user or previledged mode).
 *
 *************************************************************************/
void VIC_SetProtectionMode(VicController_t VicNumb,
                           Vic_ProtectionMode_t ProtectionType)
{
  if(VicNumb == VIC_0)
  {
    VIC0_Protection_bit.PROT = ProtectionType;
  }
  else
  {
    VIC0_Protection_bit.PROT = ProtectionType;
  }
  return;
}

/*************************************************************************
 * Function Name: VIC_GetProtectionMode
 * Parameters: VicController_t VicNumb
 * Return: Vic_ProtectionMode_t
 *
 * Description: Get the VIC access mode (previledged mode |
 *              user or previledged mode).
 *
 *************************************************************************/
Vic_ProtectionMode_t VIC_GetProtectionMode(VicController_t VicNumb)
{
  if(VicNumb == VIC_0)
  {
    return(VIC0_Protection_bit.PROT ?
            PrivilegedMode:
            UserandPrivilegedMode);
  }
  return(VIC1_Protection_bit.PROT ?
            PrivilegedMode:
            UserandPrivilegedMode);
}


/*************************************************************************
 * Function Name: VIC_Init
 * Parameters: VicController_t VicNumb
 * Return: none
 *
 * Description: Initialize VIC
 *
 *************************************************************************/
void VIC_Init(VicController_t VicNumb)
{
  SCU_PCGR0_bit.VIC = 1;    // Clock enable
  SCU_PRR0_bit.RST_VIC = 1; // Reset release

  if(VicNumb == VIC_0)
  {
    // Assign all interrupt chanels to IRQ
    VIC0_IntSelect  =  0;
    // Diasable all interrupts
    VIC0_IntEnClear = 0xFFFFFFFF;
    // Clear all software interrutps
    VIC0_SoftIntClear = 0xFFFFFFFF;
    // VIC registers can be accessed in User or privileged mode
    VIC0_Protection = 0;
    // Clear interrupt
    VIC0_CurrVectAddr = 0;
    // Clear address of the Interrupt Service routine (ISR) for non-vectored IRQs.
    VIC0_DefVectAddr = 0;

    // Clear address of the Interrupt Service routine (ISR) for vectored IRQs.
    VIC0_VectAddr0  =\
    VIC0_VectAddr1  =\
    VIC0_VectAddr2  =\
    VIC0_VectAddr3  =\
    VIC0_VectAddr4  =\
    VIC0_VectAddr5  =\
    VIC0_VectAddr6  =\
    VIC0_VectAddr7  =\
    VIC0_VectAddr8  =\
    VIC0_VectAddr9  =\
    VIC0_VectAddr10 =\
    VIC0_VectAddr11 =\
    VIC0_VectAddr12 =\
    VIC0_VectAddr13 =\
    VIC0_VectAddr14 =\
    VIC0_VectAddr15 =0;

    // Disable all vectored IRQ slots
    VIC0_VectCntl0  =\
    VIC0_VectCntl1  =\
    VIC0_VectCntl2  =\
    VIC0_VectCntl3  =\
    VIC0_VectCntl4  =\
    VIC0_VectCntl5  =\
    VIC0_VectCntl6  =\
    VIC0_VectCntl7  =\
    VIC0_VectCntl8  =\
    VIC0_VectCntl9  =\
    VIC0_VectCntl10 =\
    VIC0_VectCntl11 =\
    VIC0_VectCntl12 =\
    VIC0_VectCntl13 =\
    VIC0_VectCntl14 =\
    VIC0_VectCntl15 =0;
  }
  else
  {
    // Assign all interrupt chanels to IRQ
    VIC1_IntSelect  =  0;
    // Diasable all interrupts
    VIC1_IntEnClear = 0xFFFFFFFF;
    // Clear all software interrutps
    VIC1_SoftIntClear = 0xFFFFFFFF;
    // VIC registers can be accessed in User or privileged mode
    VIC1_Protection = 0;
    // Clear interrupt
    VIC1_CurrVectAddr = 0;
    // Clear address of the Interrupt Service routine (ISR) for non-vectored IRQs.
    VIC1_DefVectAddr = 0;

    // Clear address of the Interrupt Service routine (ISR) for vectored IRQs.
    VIC1_VectAddr0  =\
    VIC1_VectAddr1  =\
    VIC1_VectAddr2  =\
    VIC1_VectAddr3  =\
    VIC1_VectAddr4  =\
    VIC1_VectAddr5  =\
    VIC1_VectAddr6  =\
    VIC1_VectAddr7  =\
    VIC1_VectAddr8  =\
    VIC1_VectAddr9  =\
    VIC1_VectAddr10 =\
    VIC1_VectAddr11 =\
    VIC1_VectAddr12 =\
    VIC1_VectAddr13 =\
    VIC1_VectAddr14 =\
    VIC1_VectAddr15 =0;

    // Disable all vectored IRQ slots
    VIC1_VectCntl0  =\
    VIC1_VectCntl1  =\
    VIC1_VectCntl2  =\
    VIC1_VectCntl3  =\
    VIC1_VectCntl4  =\
    VIC1_VectCntl5  =\
    VIC1_VectCntl6  =\
    VIC1_VectCntl7  =\
    VIC1_VectCntl8  =\
    VIC1_VectCntl9  =\
    VIC1_VectCntl10 =\
    VIC1_VectCntl11 =\
    VIC1_VectCntl12 =\
    VIC1_VectCntl13 =\
    VIC1_VectCntl14 =\
    VIC1_VectCntl15 =0;
  }
}

/*************************************************************************
 * Function Name: VIC_GetIRQStatus
 * Parameters: VicController_t VicNumb
 * Return: Int32U
 *
 * Description: Get IRQ Status of VIC. Return register VICIRQSTATUS's value.
 *              If some IRQ interrupt request is enabled, then the
 *              corresponding bit of VICIRQSTATUS is set.
 *
 *************************************************************************/
Int32U VIC_GetIRQStatus(VicController_t VicNumb)
{
  return((VicNumb == VIC_0) ? VIC0_IRQStatus : VIC1_IRQStatus);
}

/*************************************************************************
 * Function Name: VIC_GetFIQStatus
 * Parameters: VicController_t VicNumb
 * Return: Int32U
 *
 * Description: Get FIQ Status of VIC. Return register VICFIQSTATUS's value.
 *              If some FIQ interrupt request is enabled, then the corresponding
 *              bit of VICFIQSTATUS is set. If more that one interrupt request
 *              is assigned as FIQ, then invoking this function can decide which
 *              one or ones is/are the request source(s).
 *
 *************************************************************************/
Int32U VIC_GetFIQStatus(VicController_t VicNumb)
{
  return((VicNumb == VIC_0) ? VIC0_FIQStatus : VIC1_FIQStatus);
}

/*************************************************************************
 * Function Name: VIC_EnableInt
 * Parameters: VicController_t VicNumb, Int32U IntType
 * Return: none
 *
 * Description: Enable specific interrupt
 *
 *************************************************************************/
void VIC_EnableInt(VicController_t VicNumb, Int32U IntType)
{
  if(VicNumb == VIC_0)
  {
    VIC0_IntEnable = IntType;
  }
  else
  {
    VIC1_IntEnable = IntType;
  }
}

/*************************************************************************
 * Function Name: VIC_DisableInt
 * Parameters: VicController_t VicNumb, Int32U IntType
 * Return: none
 *
 * Description: Disable specific interrupt
 *
 *************************************************************************/
void VIC_DisableInt(VicController_t VicNumb, Int32U IntType)
{
  if(VicNumb == VIC_0)
  {
    VIC0_IntEnClear = IntType;
  }
  else
  {
    VIC1_IntEnClear = IntType;
  }
}

/*************************************************************************
 * Function Name: VIC_EnableNonVectoredIRQ
 * Parameters: VicController_t VicNumb,
 *             pIRQSub - Non Vectored IRQ Sub address
 * Return: none
 *
 * Description: Set VICDefVectAddr to be the IRQ Sub address.
 *
 *************************************************************************/
void VIC_EnableNonVectoredIRQ(VicController_t VicNumb,void(*pIRQSub)())
{
  if(VicNumb == VIC_0)
  {
    VIC0_DefVectAddr = (unsigned int)pIRQSub;
  }
  else
  {
    VIC1_DefVectAddr = (unsigned int)pIRQSub;
  }
}

/*************************************************************************
 * Function Name: VIC_DisableNonVectoredIRQ
 * Parameters: VicController_t VicNumb
 * Return: none
 *
 * Description: set VICDefVectAddr to be reset value (NULL).
 *
 *************************************************************************/
void VIC_DisableNonVectoredIRQ(VicController_t VicNumb)
{
  if(VicNumb == VIC_0)
  {
    VIC0_DefVectAddr = NULL;
  }
  else
  {
    VIC1_DefVectAddr = NULL;
  }
}

/*************************************************************************
 * Function Name: VIC_SetVectoredIRQ
 * Parameters:  VicController_t VicNumb,void(*pIRQSub)(),
 *              VicIrqSlots_t VicIrqSlot, Int32U VicIntSource
 *
 *
 * Return: none
 *
 * Description:  Init vectored inerrutps
 *
 *************************************************************************/
void VIC_SetVectoredIRQ(VicController_t VicNumb,void(*pIRQSub)(),
                        VicIrqSlots_t VicIrqSlot, Int32U VicIntSource)
{
volatile unsigned long * pReg;
  // load base address of vectored address registers
  pReg = (VicNumb == VIC_0) ? &VIC0_VectAddr0 : &VIC1_VectAddr0;
  // Set Address of callback function to corresponding Slot
  *(pReg+VicIrqSlot) = (unsigned long)pIRQSub;
  // load base address of ctrl registers
  pReg = (VicNumb == VIC_0) ? &VIC0_VectCntl0 : &VIC1_VectCntl0;
  // Set source chanel and enable the slot
  *(pReg+VicIrqSlot) = VicIntSource | 0x20;
  // Clear FIQ select bit
  if(VicNumb == VIC_0)
  {
    VIC0_IntSelect &= ~(1<<VicIntSource);
  }
  else
  {
    VIC1_IntSelect &= ~(1<<VicIntSource);
  }
}

/*************************************************************************
 * Function Name: VIC_SetFastInt
 * Parameters: VicController_t VicNumb, Int32U FastIntMask
 * Return: None
 *
 * Description: Enable FIQ interrupt
 *
 *************************************************************************/
void VIC_EnaFastInt(VicController_t VicNumb, Int32U FastIntMask)
{
  if(VicNumb == VIC_0)
  {
    VIC0_IntSelect |= FastIntMask;
  }
  else
  {
    VIC1_IntSelect |= FastIntMask;
  }
}

/*************************************************************************
 * Function Name: VIC_DisFastInt
 * Parameters: VicController_t VicNumb, Int32U FastIntMask
 * Return: none
 *
 * Description: Disable FIQ interrupt
 *
 *************************************************************************/
void VIC_DisFastInt(VicController_t VicNumb, Int32U FastIntMask)
{
  if(VicNumb == VIC_0)
  {
    VIC0_IntSelect &= ~FastIntMask;
  }
  else
  {
    VIC1_IntSelect &= ~FastIntMask;
  }
}

